#!/bin/env python
# -*- coding: UTF-8 -*-
## \namespace clientes
# \brief Conjunto de clases para la gestion de los clientes Jclic.

import os
import re
import dbus

## \class manejador_Fich_Cli
# \brief Manejador de un fichero con listas de clientes
# \version 1.2.2-1
# \date 20-11-2007
class manejador_Fich_Cli:
	## \fn __init__(self,cadena)
	# \brief Constructor de la clase, fija el nombre y el path del fichero
	# de listas de clientes que encargado de gestionar
	# \param[in] self Instancia de la clase que realiza la llamada
	# \param[in] cadena Cadena de caracteres que contiene el nombre del fichero de clientes conocidos
	def __init__(self, cadena):
		self.PATH_LISTA='/usr/share/jclic/'
		self.nombre_fichero = '%s%s' % (self.PATH_LISTA, cadena)

	## \fn get_lista(self)
	# \brief Funcion para extraer la lista completa de clientes conocidos desde el fichero de lista
	# \param[in] self Instancia de la clase que realiza la llamada al metodo
	# \return Tabla con el que contine todas las filas completas del fichero de lista
	def get_lista(self):
		fichero = open('%s' % self.nombre_fichero, 'r')
		lista = fichero.readlines()
		fichero.close()
		return lista
	
	## \fn borrar_cliente_mac(self, mac)
	# \brief Funcion que elimina del fichero de clientes un cliente concreto a partir de su direccion MAC
	# \param[in] self Instancia de la clase que realiza la llamada al metodo
	# \param[in] mac Direccion MAC del elemento de la lista se quiere eliminar
	def borrar_cliente_mac(self, mac):
		patron = re.compile('%s' % mac)
		fichero = open('%s' % self.nombre_fichero, 'r')
		lista = fichero.readlines()
		fichero.close()
		fichero = open('%s' % self.nombre_fichero, 'w')
		for cliente in lista:
			if re.search(patron, cliente) == None:
				fichero.write(cliente)
		fichero.close()
		
	## \fn borrar_lista(self)
	# \brief Funcion que borra completamente el fichero de clientes
	# \param[in] self Instancia de la clase que realiza la llamada al metodo
	def borrar_lista(self):
		fichero = open('%s' % self.nombre_fichero, 'w')
		fichero.close()

	## \fn inc_mod_cliente(self, mac, ip)
	# \brief Funcion que modifica los datos de red de un cliente en caso de modificacion de su
	# direccion IP. Si el cliente no se encuentra registrado en la lista entonces se incluye como
	# nuevo
	# \param[in] self Instancia de la clase que realiza la llamada al metodo
	# \param[in] mac Direccion MAC del cliente
	# \param[in] ip Direccion IP del cliente
	# \return Valor que indica si el cliente era nuevo o ha sido actualizado
	# \retval False Nuevo cliente
	# \retval True Cliente actualizado
	def inc_mod_cliente(self, mac, ip):
		incluido = False
		fichero = open('%s' % self.nombre_fichero, 'r')
		lista = fichero.readlines()
		fichero.close()
		fichero = open('%s' % self.nombre_fichero, 'w')
		for cliente in lista:
			datos_cliente = cliente.split(';;')
			if mac == datos_cliente[0]:
				datos_cliente[1] = ip
				incluido = True
			fichero.write('%s;;%s;;%s;;\n' % (datos_cliente[0], datos_cliente[1], datos_cliente[2]))
		if incluido == False:
			fichero.write('%s;;%s;;%s;;\n' % (mac, ip, ' '))
		fichero.close()
		return incluido

	## \fn mod_info_cliente(self, mac, info)
	# \brief Funcion para modificar la informacion adicional de un cliente de la lista
	# \param[in] self Instancia de la clase que realiza la llamada al metodo
	# \param[in] mac Direccion MAC del cliente
	# \param[in] info Nueva informacion para sustituir la actual 
	def mod_info_cliente(self, mac, info):
		fichero = open('%s' % self.nombre_fichero, 'r')
		lista = fichero.readlines()
		fichero.close()
		fichero = open('%s' % self.nombre_fichero, 'w')
		for cliente in lista:
			datos_cliente = cliente.split(';;')
			if mac == datos_cliente[0]:
				datos_cliente[2] = info
			fichero.write('%s;;%s;;%s;;\n' % (datos_cliente[0], datos_cliente[1], datos_cliente[2]))
		fichero.close()

	## \fn crear_lista_sigala(self)
	# \brief Funcion para crear la lista de clientes usando el sistema sigala
	# \param[in] self Instancia de la clase que realiza la llamada al metodo
	def crear_lista_sigala(self):
		bus = dbus.SessionBus()
		sigala_jclic_dbus = bus.get_object("org.cga.sigala", "/org/cga/sigala/JClic")
		get_sigala_users_data = dbus.Interface(sigala_jclic_dbus, dbus_interface="org.cga.sigala.JClic")
		data = get_sigala_users_data.get_user_data()

		fichero = open('%s' % self.nombre_fichero, 'w')
		for user in data:
                	elemento = '%s;;%s;;%s;;\n' % (data[user]['MAC'], data[user]['IP'], data[user]['nick'])
			fichero.write(elemento)
		fichero.close()
