#!/bin/env python
# -*- coding: UTF-8 -*-
## \namespace configuracion
# \brief Conjunto de clases gestionar la configuracion y los ficheros de configuracion
# del conjunto de aplicaciones de JClic

import os
import re
import urllib

##\class gestor_Configuracion
# \brief Clase encargada de proporcionar herramientas de configuracion de las aplicaciones JClic.
# \version 1.2.2-1
# \date 21-11-2007
class gestor_Configuracion:
	##\fn __init__(self)
	# \brief Constructor de la clase. Comprueba si los directorios y ficheros de configuracion base
	# estan creados, si no lo estan crea todos los que falten
	# \param[in] self Instancia de clase que realiza la llamada
	def __init__(self):
		self.HOME = '%s' % os.environ.get('HOME')
		self.HOMECONF = '%s/JClic/' % os.environ.get('HOME')
		if self.get_dir_base() == False:
			self.set_dir_base()
			self.set_dir_projects()
			self.set_dir_bibliotecas()
			self.set_file_library()
			self.set_file_jclic_author()
			self.set_file_jclic()
			self.set_file_jclicreports()
			self.set_file_bibliotecas_actual()
		else:
			if self.get_dir_projects() == False:
				self.set_dir_projects()
				self.set_file_library()
			else:
				if self.get_file_library() == False:
					self.set_file_library()
			if self.get_dir_bibliotecas() == False:
				self.set_dir_bibliotecas()
			if self.get_file_jclic_author() == False:
				self.set_file_jclic_author()
			if self.get_file_jclic() == False:
				self.set_file_jclic()
			if self.get_file_jclicreports() == False:
				self.set_file_jclicreports()
			if self.get_file_bibliotecas_actual() == False:
				self.set_file_bibliotecas_actual()
		if self.get_file_xtecproperties() == False:
			self.set_file_xtecproperties()
	
	##\fn get_dir_base(self)
	# \brief Funcion que comprueba si existe el directorio base de configuracion de JClic dentro
	# del directorio personal de usuario
	# \param[in] self Instancia de la clase que realiza la llamada al metodo
	# \return Existencia del directorio base
	# \retval True El directorio base existe
	# \retval False El directorio base no existe
	def get_dir_base(self):
		return os.access('%s' % self.HOMECONF, os.F_OK)

	##\fn get_dir_projects(self)
	# \brief Funcion que comprueba si existe el directorio projects dentro del directorio de configuracion de JClic
	# \param[in] self Instancia de la clase que realiza la llamada al metodo
	# \return Existencia del directorio projects
	# \retval True El directorio projects existe
	# \retval False El directorio projects no existe
	def get_dir_projects(self):
		return os.access('%sprojects' % self.HOMECONF, os.F_OK)
	
	##\fn get_dir_bibliotecas(self)
	# \brief Funcion que comprueba si existe el directorio bibliotecas dentro del directorio de configuracion de JClic
	# \param[in] self Instancia de la clase que realiza la llamada al metodo
	# \return Existencia del directorio bibliotecas
	# \retval True El directorio bibliotecas existe
	# \retval False El directorio bibliotecas no existe
	def get_dir_bibliotecas(self):
		return os.access('%sbibliotecas' % self.HOMECONF, os.F_OK)

	##\fn get_file_library(self)
	# \brief Funcion que comprueba si existe el fichero library.jclic dentro del directorio projects
	# \param[in] self Instancia de la clase que realiza la llamada al metodo
	# \return Existencia del fichero library
	# \retval True El fichero library existe
	# \retval False El fichero library no existe
	def get_file_library(self):
		return os.access('%sprojects/library.jclic' % self.HOMECONF, os.F_OK)

	##\fn get_file_jclic_author(self)
	# \brief Funcion que comprueba si existe el fichero jclic_author.cfg dentro del directorio de configuracion de JClic
	# \param[in] self Instancia de la clase que realiza la llamada al metodo
	# \return Existencia del fichero jclic_author.cfg
	# \retval True El fichero jclic_author.cfg existe
	# \retval False El fichero jclic_author.cfg no existe
	def get_file_jclic_author(self):
		return os.access('%sjclic_author.cfg' % self.HOMECONF, os.F_OK)

	##\fn get_file_jclic(self)
	# \brief Funcion que comprueba si existe el fichero jclic.cfg dentro del directorio de configuracion de JClic
	# \param[in] self Instancia de la clase que realiza la llamada al metodo
	# \return Existencia del fichero jclic.cfg
	# \retval True El fichero jclic.cfg existe
	# \retval False El fichero jclic.cfg no existe
	def get_file_jclic(self):
		return os.access('%sjclic.cfg' % self.HOMECONF, os.F_OK)

	##\fn get_file_jclicreports(self)
	# \brief Funcion que comprueba si existe el fichero jclicReports.properties dentro del directorio de configuracion de JClic
	# \param[in] self Instancia de la clase que realiza la llamada al metodo
	# \return Existencia del fichero jclicReports.properties
	# \retval True El fichero jclicReports.properties existe
	# \retval False El fichero jclicReports.properties no existe
	def get_file_jclicreports(self):
		return os.access('%sjclicReports.properties' % self.HOMECONF, os.F_OK)

	##\fn get_file_bibliotecas_actual(self)
	# \brief Funcion que comprueba si existe el fichero list_bibliotecas_jclic.txt.actual dentro del directorio de configuracion de JClic
	# \param[in] self Instancia de la clase que realiza la llamada al metodo
	# \return Existencia del fichero list_bibliotecas_jclic.txt.actual
	# \retval True El fichero list_bibliotecas_jclic.txt.actual existe
	# \retval False El fichero list_bibliotecas_jclic.txt.actual no existe
	def get_file_bibliotecas_actual(self):
		return os.access('%slist_bibliotecas_jclic.txt.actual' % self.HOMECONF, os.F_OK)

	##\fn get_file_xtecproperties(self)
	# \brief Funcion que comprueba si existe el fichero .edu.xtec.properties dentro del directorio personal
	# \param[in] self Instancia de la clase que realiza la llamada al metodo
	# \return Existencia del fichero .edu.xtec.properties
	# \retval True El fichero .edu.xtec.properties existe
	# \retval False El fichero .edu.xtec.properties no existe
	def get_file_xtecproperties(self):
		return os.access('%s/.edu.xtec.properties' % self.HOME, os.F_OK)

	##\fn set_dir_base(self)
	# \brief Funcion que crea el directorio base de configuracion JClic dentro del directorio personal de usuario
	# \param[in] self Instancia de la clase que realiza la llamada al metodo
	def set_dir_base(self):
		os.mkdir('%s' % self.HOMECONF, 0755)

	##\fn set_dir_projects(self)
	# \brief Funcion que crea el directorio projects dentro del directorio de configuracion de JClic
	# \param[in] self Instancia de la clase que realiza la llamada al metodo
	def set_dir_projects(self):
		os.mkdir('%sprojects' % self.HOMECONF, 0755)

	##\fn set_dir_bibliotecas(self)
	# \brief Funcion que crea el directorio bibliotecas dentro del directorio de configuracion de JClic
	# \param[in] self Instancia de la clase que realiza la llamada al metodo
	def set_dir_bibliotecas(self):
		os.mkdir('%sbibliotecas' % self.HOMECONF, 0755)
	
	##\fn set_file_library(self)
	# \brief Funcion que crea el fichero library con la configuracion por defecto dentro del directorio projects
	# \param[in] self Instancia de la clase que realiza la llamada al metodo
	def set_file_library(self):
		fichero = open('%sprojects/library.jclic' % self.HOMECONF, 'w')
		fichero.write('<?xml version="1.0" encoding="UTF-8"?>\n')
		fichero.write('<JClicProject name="Sin_nombre" version="0.1.3" type="library">\n')
		fichero.write('\t<settings>\n')
		fichero.write('\t\t<title>Biblioteca principal</title>\n')
		fichero.write('\t\t<revision description="created" date="1/26/05" />\n')
		fichero.write('\t\t<descriptors />\n')
		fichero.write('\t</settings>\n')
		fichero.write('\t<sequence>\n')
		fichero.write('\t\t<item id="main" name="main">\n')
		fichero.write('\t\t\t<jump action="STOP" id="forward" />\n')
		fichero.write('\t\t\t<jump action="RETURN" id="back" />\n')
		fichero.write('\t\t</item>\n')
		fichero.write('\t</sequence>\n')
		fichero.write('\t<activities>\n')
		fichero.write('\t\t<activity class="@panels.Menu" name="main">\n')
		fichero.write('\t\t\t<description>\n')
		fichero.write('\t\t\t\t<p>Menú principal</p>\n')
		fichero.write('\t\t\t</description>\n')
		fichero.write('\t\t\t<messages>\n')
		fichero.write('\t\t\t\t<cell type="initial">\n')
		fichero.write('\t\t\t\t\t<style />\n')
		fichero.write('\t\t\t\t\t<p>Menú principal</p>\n')
		fichero.write('\t\t\t\t</cell>\n')
		fichero.write('\t\t\t</messages>\n')
		fichero.write('\t\t\t<settings margin="8" report="false">\n')
		fichero.write('\t\t\t\t<container bgColor="0xC0C0C0">\n')
		fichero.write('\t\t\t\t\t<counters time="false" actions="false" score="false" />\n')
		fichero.write('\t\t\t\t</container>\n')
		fichero.write('\t\t\t\t<window bgColor="0xC0C0C0" border="true" />\n')
		fichero.write('\t\t\t</settings>\n')
		fichero.write('\t\t</activity>\n')
		fichero.write('\t</activities>\n')
		fichero.write('\t<mediaBag />\n')
		fichero.write('</JClicProject>\n')
		fichero.close()			

	##\fn set_file_jclic_author(self)
	# \brief Funcion que crea el fichero jclic_author.cfg con la configuracion por defecto dentro del directorio de configuracion de JClic
	# \param[in] self Instancia de la clase que realiza la llamada al metodo
	def set_file_jclic_author(self):
		fichero = open('%sjclic_author.cfg' % self.HOMECONF, 'w')
		fichero.write('<?xml version="1.0" encoding="UTF-8"?>\n')
		fichero.write('<JClicAuthorSettings>\n')
		fichero.write('\t<language id="es" />\n')
		fichero.write('\t<paths>\n')
		fichero.write('\t\t<path id="root" path="%sprojects" />\n' % self.HOMECONF)
		fichero.write('\t</paths>\n')
		fichero.write('\t<applet codebase="http://clic.xtec.net/dist/jclic" width="700" height="450" bgcolor="0xFFFFFF" />\n')
		fichero.write('\t<sound mediaSystem="Java Media Framework" />\n')
		fichero.write('\t<lookAndFeel id="system" />\n')
		fichero.write('\t<browser id="" />\n')
		fichero.write('</JClicAuthorSettings>\n')
		fichero.close()
	
	##\fn set_file_jclic(self)
	# \brief Funcion que crea el fichero jclic.cfg con la configuracion por defecto dentro del directorio de configuracion de JClic
	# \param[in] self Instancia de la clase que realiza la llamada al metodo
	def set_file_jclic(self):
		fichero = open('%sjclic.cfg' % self.HOMECONF, 'w')
		fichero.write('<?xml version="1.0" encoding="UTF-8"?>\n')
		fichero.write('<JClicSettings>\n')
		fichero.write('\t<libraryManager autoRun="true">\n')
		fichero.write('\t\t<library name="Biblioteca principal" path="%sprojects/library.jclic" />\n' % self.HOMECONF)
		fichero.write('\t</libraryManager>\n')
		fichero.write('\t<language id="es" country="" variant="" />\n')
		fichero.write('\t<paths>\n')
		fichero.write('\t\t<path id="root" path="%sprojects" />\n' % self.HOMECONF)
		fichero.write('\t</paths>\n')
		fichero.write('\t<reporter enabled="false" class="TCPReporter" params="path=localhost:9000" />\n')
		fichero.write('\t<sound enabled="true" system="true" mediaSystem="default" />\n')
		fichero.write('\t<lookAndFeel id="system" />\n')
		fichero.write('\t<browser id="" />\n')
		fichero.write('\t<skin id="@default.xml" />\n')
		fichero.write('\t<recentFiles />\n')
		fichero.write('</JClicSettings>\n')
		fichero.close()

	##\fn set_file_jclicreports(self)
	# \brief Funcion que crea el fichero jclicReports.properties con la configuracion por defecto dentro del directorio de configuracion de JClic
	# \param[in] self Instancia de la clase que realiza la llamada al metodo
	def set_file_jclicreports(self):
		fichero = open('%sjclicReports.properties' % self.HOMECONF, 'w')
		fichero.write('#Jclic report server settings\n')
		fichero.write('#Thu Jun 08 11:26:24 CEST 2006\n')
		fichero.write('graph_color_header_bg=ADD8E6\n')
		fichero.write('verbose=true\n')
		fichero.write('graph_width=440\n')
		fichero.write('graph_color_dist=0000D0\n')
		fichero.write('graph_header_height=25\n')
		fichero.write('graph_font_size=11\n')
		fichero.write('browser=firefox\n')
		fichero.write('graph_color_border=FFFFFF\n')
		fichero.write('dbMaxConns=3\n')
		fichero.write('createTables=true\n')
		fichero.write('graph_margin_dist_y=20\n')
		fichero.write('graph_margin_dist_x=15\n')
		fichero.write('http_port=9000\n')
		fichero.write('dbServer=jdbc\:mysql\://c0/JClicReports\n')
		fichero.write('dbLogin=jclic_user\n')
		fichero.write('dbDriver=com.mysql.jdbc.Driver\n')
		fichero.write('dbMapStatements=true\n')
		fichero.write('graph_dist_width=192\n')
		fichero.write('graph_max_cols=10\n')
		fichero.write('lookAndFeel=default\n')
		fichero.write('graph_stroke_width=3.0\n')
		fichero.write('graph_color_bg=F0F8FF\n')
		fichero.write('language=es\n')
		fichero.write('graph_color_v2=00D000\n')
		fichero.write('graph_color_text=000000\n')
		fichero.write('graph_color_v1=D00000\n')
		fichero.write('tablePrefix=JCLIC_\n')
		fichero.write('autostart=true\n')
		fichero.write('graph_height=155\n')
		fichero.write('dbLogAppend=true\n')
		fichero.write('dbPassword=3cd1066666577467f39c3a0cc1c\n')
		fichero.write('graph_margin=8\n')
		fichero.write('session_lifetime=\n')
		fichero.write('graph_color_header_text=000000\n')
		fichero.write('http_timeout=1200\n')
		fichero.write('graph_margin_y=20\n')
		fichero.write('graph_font_family=Dialog\n')
		fichero.write('graph_margin_x=50\n')
		fichero.write('tooltips=true\n')
		fichero.write('graph_color_alert=FF0000\n')
		fichero.write('graph_div_y=4\n')
		fichero.write('dbMaxCheckoutSeconds=60\n')
		fichero.write('dbLogFile=jclicReportsDBConnection.log\n')
		fichero.write('timeout=1200\n')
		fichero.write('dbDebugLevel=2\n')
		fichero.write('http_autostart=true\n')
		fichero.write('dbMaxConnDays=1.0\n')
		fichero.write('dbMinConns=1\n')
		fichero.close()

	##\fn set_file_bibliotecas_actual(self)
	# \brief Funcion que crea el fichero list_bibliotecas_jclic.txt.actual vacio donde se indicaran las bibliotecas actuales
	# \param[in] self Instancia de la clase que realiza la llamada al metodo
	def set_file_bibliotecas_actual(self):
		fichero = open('%slist_bibliotecas_jclic.txt.actual' % self.HOMECONF, 'w')
		fichero.close()

	##\fn set_file_xtecproperties(self)
	# \brief Funcion que crea el fichero .edu.xtec.properties en el directorio personal
	# \param[in] self Instancia de la clase que realiza la llamada al metodo
	def set_file_xtecproperties(self):
		fichero = open('%s/.edu.xtec.properties' % self.HOME, 'w')
		fichero.write('#Wed Jan 16 11:55:45 GMT+01:00 2008\n')
		fichero.write('JClic=%s/JClic\n' % self.HOME)
		fichero.write('language=es\n')
		fichero.close()

	##\fn actualizar_librerias(self)
	# \brief Funcion que actualiza la lista de bibliotecas descargadas desde el servidor de contenidos
	# \param[in] self Instancia de la clase que realiza la llamada al metodo
	# \exception OSError Error de borrado de archivos de libreria
	# \exception IOError Error de descarga de librerias desde el servidor
	def actualizar_librerias(self):
		try:
		        urllib.urlretrieve('http://c0/jclic/list_bibliotecas_jclic.txt', '%slist_bibliotecas_jclic.txt' % self.HOMECONF)
		except IOError, msg:
		        print 'Error de conexion: %s' % msg
		        urllib.urlcleanup()
			return (1)
		
		fichero = open('%slist_bibliotecas_jclic.txt.actual' % self.HOMECONF, 'r')
		lista = fichero.readlines()
		fichero.close()
		for bib in lista:
			self.eliminar_libreria_configuracion(bib[:-1])
			try:
				os.remove('%sbibliotecas/%s' % (self.HOMECONF, bib[:-1]))
			except OSError, msg:
				print('Error de borrado:  %s' % msg)

		fichero = open('%slist_bibliotecas_jclic.txt' % self.HOMECONF, 'r')
		lista = fichero.readlines()
		fichero.close()
		for bib in lista:
	        	try:
        	        	urllib.urlretrieve('http://c0/jclic/%s' % bib[:-1], '%sbibliotecas/%s' % (self.HOMECONF, bib[:-1]))
		        except IOError, msg:
		                print 'Error de conexion: %s' % msg
	        	        urllib.urlcleanup()
				return (1)
		        urllib.urlcleanup()
			self.incluir_libreria_configuracion(bib[:-1])
		os.rename('%slist_bibliotecas_jclic.txt' % self.HOMECONF, '%slist_bibliotecas_jclic.txt.actual' % self.HOMECONF)

	##\fn incluir_libreria_configuracion(self, libreria)
	# \brief Funcion que incluye en el archivo de configuracion de librerias (jclic.cfg) una nueva libreria
	# \param[in] self Instancia de la clase que realiza la llamada al metodo
	# \param[in] libreria nombre del archivo de la nueva libreria
	def incluir_libreria_configuracion(self, libreria):
		nombre_bib = self.nombre_libreria(libreria)
		fichero = open('%sjclic.cfg' % self.HOMECONF, 'r')
		listado = fichero.readlines()
		fichero.close()
		fichero = open('%sjclic.cfg' % self.HOMECONF, 'w')
		patron = re.compile('libraryManager auto')
		for linea in listado:
			fichero.write(linea)
			if re.search(patron, linea) != None:
				fichero.write('\t\t<library name="%s" path="%sbibliotecas/%s" />\n' % (nombre_bib, self.HOMECONF, libreria))
		fichero.close()

	##\fn eliminar_libreria_configuracion(self, libreria)
        # \brief Funciona que elimina en el archivo de configuracion de librerias (jclic.cfg) una nueva libreria
        # \param[in] self Instancia de la clase que realiza la llamada al metodo
        # \param[in] libreria nombre del archivo de la libreria que se va a eliminar
        def eliminar_libreria_configuracion(self, libreria):
                nombre_bib = self.nombre_libreria(libreria)
                fichero = open('%sjclic.cfg' % self.HOMECONF, 'r')
                listado = fichero.readlines()
                fichero.close()
                fichero = open('%sjclic.cfg' % self.HOMECONF, 'w')
                patron = re.compile('%s' %nombre_bib)
                for linea in listado:
                        if re.search(patron, linea) == None:
                                fichero.write(linea)
                fichero.close()

	##\fn nombre_libreria(self, libreria)
	# \brief Funcion que obtiene el nombre de una libreria a partir del fichero de bibliotecas descargado desde el servidor
	# \param[in] self Instancia de la clase que realiza la llamada al metodo
	# \param[in] libreria nombre del fichero de biblioteca desde donde extraer su nombre
	# \return Nombre de la libreria
	# \retval Cadena Cadena de texto con el nombre de la libreria
	# \retval Desconocido Si la libreria no tiene definido un nombre
	def nombre_libreria(self, libreria):
		fichero = open('%sbibliotecas/%s' % (self.HOMECONF, libreria), 'r')
		contenido = fichero.readlines()
		fichero.close()
		patron = re.compile('title')
		for linea in contenido:
		        if re.search(patron, linea) != None:
		                print linea
		                separador = re.compile('[>*<]')
		                nombre = re.split(separador, linea)
		                sig = 0
		                for campo in nombre:
		                        if sig == 1:
						return campo
		                        if campo == 'title':
		                                sig = 1
		return 'Desconocido'

	##\fn activar_informes_player(self, ip)
	# \brief Funcion que activa el registro de informes en jclic player para conectar con el ordenador del profesor
	# \param[in] self Instancia de la clase que realiza la llamada al metodo
	# \param[in] ip Direccion del servidor de informes (equipo de profesor)
	def activar_informes_player(self, ip):
		fichero = open('%sjclic.cfg' % self.HOMECONF, 'r')
		listado = fichero.readlines()
		fichero.close()
		fichero = open('%sjclic.cfg' % self.HOMECONF, 'w')
		patron = re.compile('reporter')
		for linea in listado:
			if re.search(patron, linea) == None:
				fichero.write(linea)
			else:
				fichero.write('\t<reporter enabled="true" class="TCPReporter" params="path=%s:9000" />\n' % ip)
		fichero.close()
	
	##\fn desactivar_informes_player(self)
	# \brief Funcion que desactiva el registro de informes en jclic player para conectar con el ordenador del profesor
	# \param[in] self Instancia de la clase que realiza la llamada al metodo
	def desactivar_informes_player(self):
		fichero = open('%sjclic.cfg' % self.HOMECONF, 'r')
		listado = fichero.readlines()
		fichero.close()
		fichero = open('%sjclic.cfg' % self.HOMECONF, 'w')
		patron = re.compile('reporter')
		for linea in listado:
			if re.search(patron, linea) == None:
				fichero.write(linea)
			else:
				fichero.write('\t<reporter enabled="false" class="TCPReporter" params="path=localhost:9000" />\n')
		fichero.close()
