#!/bin/env python
# -*- coding: UTF-8 -*-
## \namespace jclic_base
# \brief Conjunto de clases bases para la ejecucion de jclic

import os
import grp
import jclic_gui

##\class jclic_Base
# \brief Clase principal de JClic donde se comprueban los usuarios, sus privilejios y se crean
# los elementos necesarios para cada usuario
# \version 1.2.2-1
# \date 28/11/2007
class jclic_Base:
	##\fn __init__(self)
	# \brief Constructor de la clase en el que se crea la interfaz de usuario correspondiente
	# a los distintos tipos de usuarios, profesorados y alumnado
	# \param[in] self Instancia de la clase que realiza la llamada
	def __init__(self):
		if self.es_profesor() == True:
			jclic = jclic_gui.jclic_Principal_Profesor('0')
		else:
			jclic = jclic_gui.jclic_Principal_Alumno()
			
	##\fn es_profesor(self)
	# \brief Funcion que comprueba si el usuario que esta ejecutando esta aplicacion pertenece
	# al grupo profesores
	# \param[in] self Instancia de la clase que realiza la llamada al metodo
	# \return Devuelve si el usuario que ejecuta esta aplicacion pertenece al grupo profesores
	# \retval True El usuario pertenece al grupo profesores
	# \retval False El usuario no pertenece al grupo profesores
	def es_profesor(self):
		gid = os.getgid()
                if self.existe_grupo_profesores() == True:
                        prof = grp.getgrnam('profesores')
                        if prof[2] == gid:
                                return True
                        else:
                                return False

	##\fn existe_grupo_profesores(self)
	# \brief Funcion que comprueba si en el sistema existe el grupo profesores
	# \param[in] self Instancia de la clase que realiza la llamada al metodo
	# \return Devuelve si existe el grupo profesores en el sistema
	# \retval True El grupo profesores existe en el sistema
	# \retval False El grupo profesores no existe en el sistema
	def existe_grupo_profesores(self):
		grupos = grp.getgrall()
                for elemento in grupos:
                        if elemento[0] == 'profesores':
                                return True
                return False
##\class jclic_Base_Sigala
# \brief Clase principal de Jclic para hacer uso del nuevo sistema sigala, ejecuta jclic como profesor
# \version 1.2.2-13
# \date 02/03/2010
class jclic_Base_Sigala:
	##\fn __init__(self)
	# \brief Constructor de la clase en la que se crea la interfaz de profesor
	# \param[in] self Instalancia de la clase que erealiza la llamada
	def __init__(self):
		jclic = jclic_gui.jclic_Principal_Profesor('1')
