#!/bin/env python
# -*- coding: UTF-8 -*-
##\namespace jclic_comandos
# \brief Conjunto de clases encargadas de ejecutar los comandos jclic del sistema

import os
import comunicaciones
import configuracion

##\class ejecutor_Jclic
# \brief Clase para la definicion y ejecucion de cada una de las herramientas de JClic
# \version 1.2.2-1
# \date 28/11/2007
class ejecutor_Jclic:
	##\fn __init__(self)
	# \brief Constructor de la clase 
	# \param[in] self Instancia de la clase que realiza la llamada
	def __init__(self):
		self.P_JCLIC = '/usr/local/jclic/jclicapp/'
		self.P_JAVA = '/usr/bin/java'
		self.P_TMP = '/tmp/%d_jclic_status' % os.getuid()
		self.P_SOUND = '/usr/local/jclic/sound_jclicplayer'
		self.comunicador = comunicaciones.EBCe()

	##\fn ejecutor_mensaje(self, mensaje, ip)
	# \brief Funcion que realiza la operacion indicada en el mensaje obtenido desde el servidor
	# de informes
	# \param[in] self Instancia de la clase que realiza la llamada al metodo
	# \param[in] mensaje Mensaje obtenido del servidor de informes
	# \param[in] ip Direccion IP del servidor de informes
	def ejecutor_mensaje(self, mensaje, ip):
		if mensaje == 'mensaje0003r':
			fichero = open('%s' % self.P_TMP, 'r')
			estado = fichero.read()
			fichero.close()
			if estado == '0':
				mensaje = 'mensaje0010r\0'
			elif estado == '1':
				mensaje = 'mensaje0011r\0'
			elif estado == '2':
				mensaje = 'mensaje0012r\0'
			else:
				print "Estado de ejecucion incorrecto"
			self.comunicador.enviar_mensaje_EBCe(ip, mensaje, 60001)
		elif mensaje == 'mensaje0006r':
			self.configurador = configuracion.gestor_Configuracion()
			self.configurador.actualizar_librerias()	
			self.ejecutor_jclicplayer_informes(ip)

	##\fn ejecutor_jclicauthor(self)
	# \brief Funcion que ejecuta JClic Author
	# \param[in] self Instancia de la clase que realiza la llamada al metodo
	def ejecutor_jclicauthor(self):
		os.spawnlp(os.P_WAIT, '%s' % self.P_JAVA, '%s' % self.P_JAVA, '-jar', '%sjclicauthor.jar' % self.P_JCLIC)

	##\fn ejecutor_jclicplayer(self)
	# \brief Funcion que ejecuta JClic Player sin soporte para informes
	# \param[in] self Instancia de la clase que realiza la llamada al metodo
	def ejecutor_jclicplayer(self):
		self.cambiar_estado_ejecucion('0')
		configurador = configuracion.gestor_Configuracion()
		configurador.actualizar_librerias()	
		os.spawnlp(os.P_WAIT, '%s' % self.P_SOUND, '%s' % self.P_SOUND)
		self.cambiar_estado_ejecucion('1')

	##\fn ejecutor_jclicplayer_informes(self, ip)
	# \brief Funcion que ejecuta JClic Player con soporte para informes, conectandose al servidor de
	# informes (ordenador del profesor) para guardar los resultados
	# \param[in] self Instancia de la clase que realiza la llamada al metodo
	# \param[in] ip Direccion IP del servidor informes
	def ejecutor_jclicplayer_informes(self, ip):
		com = comunicaciones.EBCe()
		com.enviar_mensaje_EBCe(ip, 'mensaje0012r\0', 60001)
		self.configurador.activar_informes_player(ip)
		self.cambiar_estado_ejecucion('2')
		os.spawnlp(os.P_WAIT, '%s' % self.P_JAVA, '%s' % self.P_JAVA, '-jar', '%sjclicplayer.jar' % self.P_JCLIC)
		self.configurador.desactivar_informes_player()
		self.cambiar_estado_ejecucion('1')
		com = comunicaciones.EBCe()
		com.enviar_mensaje_EBCe(ip, 'mensaje0011r\0', 60001)
	
	##\fn ejecutor_jclicreports(self)
	# \brief Funcion que ejecuta JClic Reports
	# \param[in] self Instancia de la clase que realiza la llamada
	# \return Pid del proceso en el que se ejecuto jclicreports
	def ejecutor_jclicreports(self):
		pid = os.spawnlp(os.P_NOWAIT, '%s' % self.P_JAVA, '%s' % self.P_JAVA, '-jar', '%sreportServer.jar' % self.P_JCLIC)
		return pid

	##\fn cambiar_estado_ejecucion(self, estado)
	# \brief Funcion para cambiar el estado de ejecucion de jclic en el cliente
	# Para la consulta del estado actual (mensaje0003r) se definen los siguientes estados
	# - 0  El demonio no se esta ejecutando
	# - 1  El demonio se esta ejecutando y espara envios del servidor
	# - 2  Se esta ejecutando JClic Player con soporte de informes
	# \param[in] self Instancia de la clase que realiza la llamada al metodo
	# \param[in] estado Nuevo estado de ejecucion
	def cambiar_estado_ejecucion(self, estado):
		fichero = open('%s' % self.P_TMP, 'w')
		fichero.write(estado)
		fichero.close()
