#!/bin/env python
# -*- coding: UTF-8 -*-
## \namespace jclic_gui
# \brief Conjunto de clases que compenen todos los componentes y las interfaces de usuario
# para el acceso a las herramientas jclic

import string
import re
import socket
import time
from threading import Thread
import thread
import threading
import os
import signal
import pygtk
pygtk.require('2.0')
import gtk
import comunicaciones
import configuracion
import clientes
import jclic_comandos
import urllib

##\class jclic_Principal
# \brief Clase base general para la creacion de una interfaz de usuario para jclic
# \version 1.2.2-1
# \date 28/11/2007
class jclic_Principal:
	##\fn __init__(self)
	# \brief Constructor de la clase. Crea la interfaz de usuario y todos los elementos necesario para
	# el uso de las herramientas JClic
	# \param[in] self Instancia de la clase que realiza la llamada
	def __init__(self):
		self.PATH = '/usr/local/jclic/'
		self.configurador = configuracion.gestor_Configuracion()
		self.crear_interfaz()
		
	##\fn crear_interfaz(self)
	# \brief Metodo para crear la interfaz de usuario
	# \param[in] self Instancia de la clase que realiza la llamada al metodo
	def crear_interfaz(self):
		self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
		self.window.set_border_width(10)
		self.window.set_icon_from_file('%simg/jclic_icon.png' % self.PATH)
		self.window.set_title('Herramientas JClic')
		self.window.set_resizable(False)
		self.box = gtk.VBox(False, 10)
		self.window.add(self.box)
		self.box.show()

		self.window.connect("delete_event", self.delete_event)
                self.window.connect("destroy", self.destroy)
	
	##\fn delete_event(self, widget, data = None)
	# \brief Funcion para terminar la ejecucion de la aplicacion
	# \param[in] self Instancia de clase que realiza la llamada
	# \param[in] widget Widget asociado al evento
	# \param[in] data datos pasados a la funcion	
	def delete_event(self, widget, data = None):
		gtk.main_quit()

	##\fn destroy(self, widget, data = None)
	# \brief Funcion para terminar la ejecucion de la aplicacion	
	# \param[in] self Instancia de clase que realiza la llamada
	# \param[in] widget Widget asociado al evento
	# \param[in] data datos pasados a la funcion	
	def destroy(self, widget, data = None):
		gtk.main_quit()

##\class jclic_Principal_Profesor
# \brief Clase hija de la clase base de interfaz de jclic para construir la interfaz de usuario para el
# profesorado
# \version 1.2.2-1
# \date 30/10/2007
class jclic_Principal_Profesor(jclic_Principal):
	##\fn __init__(self)
	# \brief Constructor de la clase, llama al contructor de la clase padre y ajusta su opciones y componentes locales
	# \param[in] self Instancia de la clase que realiza la llamada
	# \param[in] sigala Indica si la aplicacion se ejecuta desde sigala o desde el sistema
	def __init__(self, sigala):
		jclic_Principal.__init__(self)
		self.window.set_size_request(650, 400)
		self.window.show()
		self.herramienta = herramienta_Author(self.window)
		self.box.pack_start(self.herramienta.bloque_frame, False, False, 0)
		self.herramienta2 = herramienta_Player(self.window)
		self.box.pack_start(self.herramienta2.bloque_frame, False, True, 0)
		self.herramienta3 = herramienta_Player_Informe(self.window)
		self.box.pack_start(self.herramienta3.bloque_frame, False, True, 0)
		self.herramienta4 = herramienta_Reports(self.window, sigala)
		self.box.pack_start(self.herramienta4.bloque_frame, False, True, 0)

##\class jclic_Principal_Alumno
# \brief Interfaz principal de JClic con las herramientas de alumno
# \version 1.2.2-1
# \date 28/11/2007
class jclic_Principal_Alumno(jclic_Principal):
	##\fn __init__(self)
	# \brief Constructor de la clase, llama al contructor de la clase padre y ajusta su opciones y componentes locales
	# \param[in] self Instancia de la clase que realiza la llamada
	def __init__(self):
		self.HOMECONF = '%s/JClic/' % os.environ.get('HOME')
		jclic_Principal.__init__(self)
		self.window.set_size_request(650, 200)
		self.window.show()
		try:
                        urllib.urlretrieve('http://c0/jclic/activar_author.txt', '%sactivar_author.txt' % self.HOMECONF)
                        actualizado = True
                except IOError, msg:
                        print 'Error de conexion: %s' % msg
                        urllib.urlcleanup()
                        actualizado = False
                if actualizado == True:
                        fichero = open('%sactivar_author.txt' % self.HOMECONF, 'r')
                        dato = fichero.readline()
                        fichero.close()
                        if dato == '1\n':
                                self.herramienta = herramienta_Author(self.window)
                                self.box.pack_start(self.herramienta.bloque_frame, False, False, 0)
                                self.window.set_size_request(650, 300)
		self.herramienta2 = herramienta_Player(self.window)
		self.box.pack_start(self.herramienta2.bloque_frame, False, True, 0)
		self.herramienta3 = herramienta_Player_Informe(self.window)
		self.box.pack_start(self.herramienta3.bloque_frame, False, True, 0)


##\class herramientas_Jclic
# \brief Clase para crear los componentes de herramientas, widgets para ejecutar las herramientas jclic
# \version 1.2.2-1
# \date 28/11/2007
class herramientas_Jclic:
	##\fn __init__(self, win)
	# \brief Constructor de la clase base para la creacion de las distintas herramientas de JClic
	# \param[in] self Instancia de la clase que realiza la llamada
	# \param[in] win Referencia a la ventana que contiene al controlador
	def __init__(self, win):
		self.PATH = '/usr/local/jclic/'
		self.window = win

		self.bloque_frame = gtk.Frame()
		self.bloque_frame.set_shadow_type(gtk.SHADOW_NONE)
		self.bloque_frame.set_size_request(530, 85)
		self.bloque_frame.show()
		self.bloque_fixed = gtk.Fixed()
		self.bloque_frame.add(self.bloque_fixed)
		self.bloque_fixed.show()

		self.boton = gtk.Button()
		self.boton.set_size_request(80, 60)
		self.bloque_fixed.put(self.boton, 10, 10)
		self.boton.show()

		self.titulo = gtk.Label()
		self.titulo.set_use_markup(True)
		self.bloque_fixed.put(self.titulo, 110, 10)
		self.titulo.show()
		self.explicacion = gtk.Label()
		
		self.bloque_fixed.put(self.explicacion, 120, 30)
		self.explicacion.show()

##\class herramienta_Author
# \brief Clase hija de herramientas_Jclic que implementa el controlador para Jclic Author
# \version 1.2.2-1
# \date 03/12/2007
class herramienta_Author(herramientas_Jclic):
	##\fn __init__(self, win)
	# \brief Constructor de la clase, llama al constructor de la clase padre y fija las opciones correspondientes para author
	# \param[in] self Instancia de la clase que realiza la llamada
	# \param[in] win Referencia a la ventana que contiene al controlador
	def __init__(self, win):
		herramientas_Jclic.__init__(self, win)
		self.imagen = gtk.Image()
		self.imagen.set_from_file('%simg/jclicauthor.png' % self.PATH)
		self.boton.set_image(self.imagen)
		self.boton.connect("clicked", self.ejecutar_author, None)
		self.titulo.set_markup('<b>JClic Author</b>')
		self.explicacion.set_text('Herramienta de JClic con la que se pueden crear nuevos proyectos\nde actividades e incluso editar proyectos ya existentes para adaptarlos\na nuestras necesidades.')

	##\fn ejecutar_author(self, widget, data = None)
	# \brief Funcion encargada de ejecutar JClic Author
	# \param[in] self Instancia de la clase que realiza la llamada al metodo
	# \param[in] widget Widget asociado al evento
	# \param[in] data Datos pasados a la funcion
	def ejecutar_author(self, widget, data = None):
		ejecutor = jclic_comandos.ejecutor_Jclic()
		ejecutor.ejecutor_jclicauthor()

##\class herramienta_Player
# \brief Clase hija de herramientas_Jclic que implementa el controlador para Jclic Player sin soporte de informes
# \version 1.2.2-1
# \date 03/12/2007
class herramienta_Player(herramientas_Jclic):
	##\fn __init__(self, win)
	# \brief Constructor de la clase, llama al constructor de la clase padre y fija las opciones correspondientes para player sin informes
	# \param[in] self Instancia de la clase que realiza la llamada
	# \param[in] win Referencia a la ventana que contiene al controlador
	def __init__(self, win):
		herramientas_Jclic.__init__(self, win)
		self.imagen = gtk.Image()
		self.imagen.set_from_file('%simg/jclic.png' % self.PATH)
		self.boton.set_image(self.imagen)
		self.boton.connect("clicked", self.ejecutar_player, None)
		self.titulo.set_markup('<b>JClic Player</b>')
		self.explicacion.set_text('Herramienta JClic con la que se pueden realizar y utilizar actividades\ndesde las bibliotecas instaladas en el centro o cualquier otra actividad\nexterna. Esta herramienta no almacena los resultados obtenidos.')

	##\fn ejecutar_player(self, widget, data = None)
	# \brief Funcion encargada de ejecutar JClic Player
	# \param[in] self Instancia de la clase que realiza la llamada al metodo
	# \param[in] widget Widget asociado al evento
	# \param[in] data Datos pasados a la funcion
	def ejecutar_player(self, widget, data = None):
		ejecutor = jclic_comandos.ejecutor_Jclic()
		ejecutor.ejecutor_jclicplayer()

##\class herramienta_Player_Informe
# \brief Clase hija de herramientas_Jclic que implementa el controlador para Jclic Player con soportes para informe
# \version 1.2.2-1
# \date 03/12/2007
class herramienta_Player_Informe(herramientas_Jclic):
	##\fn __init__(self, win)
	# \brief Constructor de la clase, llama al constructor de la clase padre y fija las opciones correspondientes para player con informes
	# \param[in] self Instancia de la clase que realiza la llamada
	# \param[in] win Referencia a la ventana que contiene al controlador
	def __init__(self, win):
		herramientas_Jclic.__init__(self, win)
		self.imagen = gtk.Image()
		self.imagen.set_from_file('%simg/jclicplayerinforme.png' % self.PATH)
		self.boton.set_image(self.imagen)
		self.boton.connect("clicked", self.ejecutar_playerinforme, None)
		self.titulo.set_markup('<b>Conectar con servidor de informes</b>')
		self.explicacion.set_text('Herramienta para conectar con el servidor de informes del aula y poder\nregistrar los resultados obtenidos en la realizacion de actividades JClic.')

	##\fn ejecutar_playerinforme(self, widget, data = None)
	# \brief Metedo asociado al click del boton de la herramienta, se encarga de mostrar la interfaz para conectarse a un servidor de informes
	# \param[in] self Instancia de la clase que realiza la llamada al metodo
	# \param[in] widget Widget asosciado al evento relacionado con el metodo
	# \param[in] data Datos pasados al metodo
	def ejecutar_playerinforme(self, widget, data = None):
		self.window.set_sensitive(False)
		self.gest_clientes = jclic_Player_Informes(self.window)

##\class servidor_Informes_Clientes
##\class herramienta_Reports
# \brief Clase hija de herramientas_Jclic que implementa el controlador para Jclic Reports
# \version 1.2.2-1
# \date 03/12/2007
class herramienta_Reports(herramientas_Jclic):
	##\fn __init__(self, win)
	# \brief Constructor de la clase, llama al constructor de la clase padre y fija las opciones correspondientes para reports
	# \param[in] self Instancia de la clase que realiza la llamada
	# \param[in] win Referencia a la ventana que contiene al controlador
	# \param[in] sigala Parametro que indica si ha sido ejecutado o no desde sigala
	def __init__(self, win, sigala):
		self.sigala = sigala
		herramientas_Jclic.__init__(self, win)
		self.imagen = gtk.Image()
		self.imagen.set_from_file('%simg/reports.png' % self.PATH)
		self.boton.set_image(self.imagen)
		self.boton.connect("clicked", self.ejecutar_reports, None)
		self.titulo.set_markup('<b>Servidor de informes</b>')
		self.explicacion.set_text('Herramienta JClic para activar el servidor de informes y los ordenadores\ndel aula puedan guardar los resultados obtenidos en la ralizacion de\nactividades JClic.')

	##\fn ejecutar_reports(self, widget, data = None)
	# \brief Metedo asociado al click del boton de la herramienta, se encarga de ejecutar el jclic reports y el gestor de clientes
	# \param[in] self Instancia de la clase que realiza la llamada al metodo
	# \param[in] widget Widget asosciado al evento relacionado con el metodo
	# \param[in] data Datos pasados al metodo
	def ejecutar_reports(self, widget, data = None):
		ejec_reports = jclic_comandos.ejecutor_Jclic()
		pid_reports = ejec_reports.ejecutor_jclicreports()
		time.sleep(2.0)
		self.window.set_sensitive(False)
		self.gest_clientes = servidor_Informes_Clientes(self.window, pid_reports, self.sigala)

##\class servidor_Informes_Clientes
# \brief Clase para crear la herramienta necesaria para gestionar y controlar a los clientes
# del aula donde se ejcetuca el servidor de informes. Permite abrir en los clientes el jclic
# player para que almacenen sus resultados automaticamente.
# \version 1.2.2-1
# \date 10/12/2007
class servidor_Informes_Clientes:
	##\fn __init__(self, win)
	# \brief Constructor de la clase para crear la interfaz de la herramienta desde donde se gestionan los usuarios del aula
	# \param[in] self Instancia de la clase que realiza la llamada
	# \param[in] win Referencia a la ventana base de la aplicacion
	# \param[in] pid_reports PID del proceso de ejecucion de Jclic Reports
	# \param[in] sigala Parametro que indica si la aplicacion se ejecuta desde sigala o no
	def __init__(self, win, pid_reports, sigala):
		self.ejecutandose = True
		self.pid_r = pid_reports
		self.PATH = '/usr/local/jclic/'
		self.local_ip = self.get_Direccion_IP_Local()
		self.window_base = win
		if sigala == '0':
			self.gestor_lista = clientes.manejador_Fich_Cli('clientes_jclic.txt')
		else:
			self.gestor_lista = clientes.manejador_Fich_Cli('clientes_jclic_sigala.txt')
			self.gestor_lista.crear_lista_sigala()

		self.imagen_jclic_estado = gtk.Image()
		self.imagen_jclic_estado = gtk.gdk.pixbuf_new_from_file('%simg/jclicplayer_cerrar_icon.png' % self.PATH)
		self.imagen_conexion_estado = gtk.Image()
		self.imagen_conexion_estado = gtk.gdk.pixbuf_new_from_file('%simg/no_network_local_icon.png' % self.PATH)
	
		self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
		self.window.set_border_width(10)
		self.window.set_icon_from_file('%simg/jclic_icon.png' % self.PATH)
		self.window.set_title('Servidor de informes JClic - Gestor de clientes')
		self.window.set_resizable(False)
		self.window.set_size_request(650, 500)
		self.box = gtk.VBox(False, 10)
		self.window.add(self.box)
		self.box.show()

		# Se crea el contenedor de la lista
                self.listado_fixed = gtk.Fixed()
                self.listado_fixed.set_size_request(365, 380)
                self.et_lista = gtk.Label()
                self.et_lista.set_markup("<b>Clientes conocidos</b>")
                self.et_lista.set_use_markup(True)
                self.listado_fixed.put(self.et_lista, 0, 0)
                self.et_lista.show()
		self.et_dir_serv = gtk.Label()
		self.et_dir_serv.set_markup("<b>Dirección IP del servidor de informes: %s</b>" % self.get_Direccion_IP_Local())
                self.et_dir_serv.set_use_markup(True)
		self.listado_fixed.put(self.et_dir_serv, 220, 0)
		self.et_dir_serv.show()
                self.contenedor_lista = gtk.ScrolledWindow()
                self.contenedor_lista.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
                self.contenedor_lista.set_shadow_type(gtk.SHADOW_IN)
                self.listado = gtk.ListStore(gtk.gdk.Pixbuf, str, gtk.gdk.Pixbuf, str, str, str, str)
		self.cargar_listado()
                self.vista_arbol_lista = gtk.TreeView()
                self.vista_arbol_lista.set_model(self.listado)

		# Columna de estado de la lista (icono)
                self.contenido_columna_estado = gtk.CellRendererPixbuf()
                self.columna_estado = gtk.TreeViewColumn('Estado\nconexión', self.contenido_columna_estado, pixbuf=0)
                self.vista_arbol_lista.append_column(self.columna_estado)

		# Columna de estado de la lista (texto)
                self.contenido_columna_estado_valor = gtk.CellRendererText()
                self.columna_estado_valor = gtk.TreeViewColumn('Estado conexión valor', self.contenido_columna_estado_valor, text=1)
                self.vista_arbol_lista.append_column(self.columna_estado_valor)
                self.columna_estado_valor.set_visible(False)

		# Columna de uso de la lista (icono)
                self.contenido_columna_uso = gtk.CellRendererPixbuf()
                self.columna_uso = gtk.TreeViewColumn('Estado\nJClic', self.contenido_columna_uso, pixbuf=2)
                self.vista_arbol_lista.append_column(self.columna_uso)

		# Columna de uso de la lista (texto)
                self.contenido_columna_uso_valor = gtk.CellRendererText()
                self.columna_uso_valor = gtk.TreeViewColumn('Estado JClic valor', self.contenido_columna_uso_valor, text=3)
                self.vista_arbol_lista.append_column(self.columna_uso_valor)
                self.columna_uso_valor.set_visible(False)

                # Columna de direccion ip de la lista
                self.contenido_columna_dirip = gtk.CellRendererText()
                self.columna_dirip = gtk.TreeViewColumn('Dirección IP', self.contenido_columna_dirip, text=4)
                self.vista_arbol_lista.append_column(self.columna_dirip)

                # Columna de informacion de la lista
                self.contenido_columna_informacion = gtk.CellRendererText()
                self.columna_informacion = gtk.TreeViewColumn('Información', self.contenido_columna_informacion, text=5)
                self.vista_arbol_lista.append_column(self.columna_informacion)

                # Columna de MAC de la lista
                self.contenido_columna_mac = gtk.CellRendererText()
                self.columna_mac = gtk.TreeViewColumn('MAC', self.contenido_columna_mac, text=6)
                self.vista_arbol_lista.append_column(self.columna_mac)
                self.columna_mac.set_visible(False)

                # Configuracion de opciones de visibilidad de la lista
                self.seleccion_vista = self.vista_arbol_lista.get_selection()
                self.seleccion_vista.set_mode(gtk.SELECTION_SINGLE)

		# Dibujar la lista en la ventana principal
                self.listado_fixed.put(self.contenedor_lista, 0, 20)
                self.contenedor_lista.add(self.vista_arbol_lista)
                self.contenedor_lista.set_size_request(630, 360)
                self.contenedor_lista.show_all()
                self.box.pack_start(self.listado_fixed, True, True, 0)
		self.listado_fixed.show()

		#Botonera de la ventana
		self.botonera_fixed = gtk.Fixed()
		self.boton_ejecutar_todos = gtk.Button("Ejecutar todos")
		self.boton_ejecutar_todos.connect("clicked", self.ejecutar_todos_clientes)
		self.boton_ejecutar_todos.set_size_request(160, 30)
		self.boton_ejecutar_todos.show()
		self.botonera_fixed.put(self.boton_ejecutar_todos, 0, 0)

		self.boton_ejecutar_sel = gtk.Button("Ejecutar seleccionado")
		self.boton_ejecutar_sel.connect("clicked", self.ejecutar_cliente_sel)
		self.boton_ejecutar_sel.set_size_request(160, 30)
		self.boton_ejecutar_sel.show()
		self.botonera_fixed.put(self.boton_ejecutar_sel, 0, 35)

		self.boton_borrar_lista = gtk.Button("Borrar lista")
		self.boton_borrar_lista.connect("clicked", self.borrar)
		self.boton_borrar_lista.set_size_request(160, 30)
		self.boton_borrar_lista.show()
		self.botonera_fixed.put(self.boton_borrar_lista, 165, 0)

		self.boton_borrar_sel = gtk.Button("Borrar seleccionado")
		self.boton_borrar_sel.connect("clicked", self.borrar)
		self.boton_borrar_sel.set_size_request(160, 30)
		self.boton_borrar_sel.show()
		self.botonera_fixed.put(self.boton_borrar_sel, 165, 35)

		self.boton_editar = gtk.Button("Editar seleccionado")
		self.boton_editar.connect("clicked", self.editar_cliente)
		self.boton_editar.set_size_request(160, 30)
		self.boton_editar.show()
		self.botonera_fixed.put(self.boton_editar, 330, 35)

		self.boton_ayuda = gtk.Button("Ayuda")
		self.boton_ayuda.connect("clicked", self.mostrar_ayuda)
		self.boton_ayuda.set_size_request(160, 30)
		self.boton_ayuda.show()
		self.botonera_fixed.put(self.boton_ayuda, 330, 0)

		self.boton_ayuda = gtk.Button("Revisar informes")
		self.boton_ayuda.connect("clicked", self.mostrar_informes)
		self.boton_ayuda.set_size_request(135, 65)
		self.boton_ayuda.show()
		self.botonera_fixed.put(self.boton_ayuda, 495, 0)
		self.box.pack_start(self.botonera_fixed, True, True, 0)
		self.botonera_fixed.show()

		self.window.connect("delete_event", self.delete_event)
		self.window.connect("delete_event", self.delete_event)
                self.window.connect("destroy", self.destroy)
		self.window.show()
		self.hilo_sock = thread.start_new_thread(self.despacho_clientes,())
		self.hilo_comprobacion = thread.start_new_thread(self.ejecutar_comprobacion,())

	##\fn delete_event(self, widget, data = None)
	# \brief Funcion para terminar la ejecucion de la aplicacion
	# \param[in] self Instancia de clase que realiza la llamada
	# \param[in] widget Widget asociado al evento
	# \param[in] data datos pasados a la funcion	
	def delete_event(self, widget, data = None):
		os.kill(self.pid_r, signal.SIGKILL)
		self.ejecutandose = False
		envio = comunicaciones.EBCe()
		envio.enviar_mensaje_EBCe(self.local_ip, 'mensajeXXXXr\0', 60001)
		self.window_base.set_sensitive(True)

	##\fn destroy(self, widget, data = None)
	# \brief Funcion para terminar la ejecucion de la aplicacion	
	# \param[in] self Instancia de clase que realiza la llamada
	# \param[in] widget Widget asociado al evento
	# \param[in] data datos pasados a la funcion	
	def destroy(self, widget, data = None):
		os.kill(self.pid_r, signal.SIGKILL)
		self.ejecutandose = False
		envio = comunicaciones.EBCe()
		envio.enviar_mensaje_EBCe(self.local_ip, 'mensajeXXXXr\0', 60001)
		self.window_base.set_sensitive(True)

	##\fn cargar_listado(self)
	# \brief Funcion que carga el listado de clientes conocido en la ventana
	# \param[in] self Instancia de la clase que realiza la llamada al metodo
	def cargar_listado(self):
		self.listado.clear()
		clientes = self.gestor_lista.get_lista()
		for cliente in clientes:
			datos_cliente = cliente.split(';;')
			print datos_cliente[0]
			print datos_cliente[1]
			print datos_cliente[2]
			cadena = '0'
			self.listado.append([self.imagen_conexion_estado, cadena, self.imagen_jclic_estado, cadena, datos_cliente[1], datos_cliente[2], datos_cliente[0]])
			ip = datos_cliente[1]
			self.hilo_actualiza_estado_conexion = thread.start_new_thread(self.comprobar_Conexion_Cliente, (datos_cliente[1], datos_cliente[0]))

	##\fn despacho_clientes(self)
	# \brief Funcion para recepcionar las conexiones y mensajes de los clientes y realizar las
	# los operaciones necesarias por cada mensaje
	# \param[in] self Instancia de la clase que realiza la llamada al metodo
	def despacho_clientes(self):
		self.conector_entrante = comunicaciones.EBCr()
		self.conector_entrante.abrir_socket_puerto(60001)
		while 1:
			msg, ip, mac = self.conector_entrante.leer_EBCr()
			if msg == 'mensajeXXXXr\0' and ip == self.local_ip:
				time.sleep(1)
				self.conector_entrante.cerrar_socket()
				thread.exit()
			self.tratamiento_mensaje_entrante(msg, ip, mac)

	##\fn tratamiento_mensaje_entrante(self, msg, ip, mac)
	# \brief Funcion para tratar los mensajes entrantes en el servidor de informes realizando las operaciones oportunas
	# Tipos de mensajes:
	# - 'mensajeXXXXr' Terminar la ejecicion del hilo del socket
	# - 'mensaje0001r' Nuevo cliente o actulizacion de datos
	# - 'mensaje0003r' Peticion de estado de ejecucion de JClic Player con soporte de informes (clientes)
	# - 'mensaje0004r' Cliente conectado a la espera de empezar a ejecutar JClic Player
	# - 'mensaje0005r' Cliente con conexion a la red
	# - 'mensaje0006r' Ejecutar player en los clientes
	# - 'mensaje0010r' El cliente no esta esperando el envio desde del servidor de informes para JClic Player
	# - 'mensaje0011r' El cliente se encuentra a la espera de envido desde el servidor de informes
	# - 'mensaje0012r' El cliente se encuentra ejecuntado JClic Player
	# \param[in] self Instancia de la clase que realiza la llamada al metodo
	# \param[in] msg Mensaje de entrada
	# \param[in] ip Direccion IP del emisor del mensaje
	# \param[in] mac Direccion MAC del elmisor del mensaje
	def tratamiento_mensaje_entrante(self, msg, ip, mac):
		if msg == 'mensaje0001r\0':
			self.listar_Nuevo_Cliente(ip, mac)
		elif msg == 'mensaje0010r\0':
			self.set_Flag_Ejecucion_Remota(mac, 0) 
		elif msg == 'mensaje0011r\0':
			self.set_Flag_Ejecucion_Remota(mac, 1)
		elif msg == 'mensaje0012r\0':
			self.set_Flag_Ejecucion_Remota(mac, 2)
		else:
			print "Mensaje de recepcion incorrecto"

	##\fn listar_Nuevo_Cliente(self, ip, mac)
	# \brief Funcion para incluir en la lista de clientes conocido un cliente nuevo o actualizar sus datos de red (direccion IP)
	# tanto en el fichero de listado como en el listado grafico de la interfaz de usuario
	# \param[in] self Instancia de la clase que realiza la llamada al metodo
	# \param[in] ip Direccion IP del cliente remoto
	# \param[in] mac Direccion MAC del cliente remoto
	def listar_Nuevo_Cliente(self, ip, mac):
		existente = self.gestor_lista.inc_mod_cliente(mac, ip)
		imagen = gtk.Image()
		imagen = gtk.gdk.pixbuf_new_from_file('%simg/network_local_icon.png' % self.PATH)
		if existente == False:
			self.listado.append([self.imagen_conexion_estado, '0', self.imagen_jclic_estado, '0', ip, '', mac])
		else:
			actualizado = False
			treeiter = self.listado.get_iter_first()
			while actualizado == False:
				mac_actual = self.listado.get_value(treeiter, 6)
				if mac_actual == mac:
					self.listado.set_value(treeiter, 4, ip)
					actualizado = True
				else:
					treeiter = self.listado.iter_next(treeiter)
					if treeiter == None:
						break

	##\fn set_Flag_Ejecucion_Remota(self, mac, estado)
	# \brief Funcion para modificar el estade de JClic del cliente  correspondiente en la lista de clientes conocidos
	# Estados de entrada:
	#  - '0' Cliente no ejecuta se encuentra a la espera de ejecucion
	#  - '1' Cliente a la espera de ejecucion
	#  - '2' Cliente ejecutando JClic Player
	# \param[in] self Instancia de la clase que realiza la llamada al metodo
	# \param[in] mac Direccion MAC del cliente que a modificado su estado
	# \param[in] estado Nuevo estado del cliente
	def set_Flag_Ejecucion_Remota(self, mac, estado):
		print "ENTRANDO"
		imagen = gtk.Image()
		actualizado = False
		treeiter = self.listado.get_iter_first()
		while actualizado == False:
			mac_actual = self.listado.get_value(treeiter, 6)
			if mac == mac_actual:
				if estado == 0:
					imagen = gtk.gdk.pixbuf_new_from_file('%simg/jclicplayer_cerrar_icon.png' % self.PATH)
					self.listado.set_value(treeiter, 2, imagen)
					self.listado.set_value(treeiter, 3, '0')
				elif estado == 1:
					imagen = gtk.gdk.pixbuf_new_from_file('%simg/jclicejecutando_icon.png' % self.PATH)
					self.listado.set_value(treeiter, 2, imagen)
					self.listado.set_value(treeiter, 3, '1')
				elif estado == 2:
					imagen = gtk.gdk.pixbuf_new_from_file('%simg/jclicplayerinforme_icon.png' % self.PATH)
					self.listado.set_value(treeiter, 2, imagen)
					self.listado.set_value(treeiter, 3, '2')
				else:
					print "Estado de ejecucion incorrecto"
				actualizado = True

			else:
				treeiter = self.listado.iter_next(treeiter)
				if treeiter == None:
					break

	##\fn get_Direccion_IP_Local(self)
	# \brief Funcion que develve la direccion IP local del equipo que ejecuta la aplicacion
	# \param[in] self Instancia de la clase que realiza la llamada al metodo
	# \return Direccion IP local del equipo
	def get_Direccion_IP_Local(self):
		comando = '/sbin/ifconfig | grep "inet:" | grep -v "127.0.0.1" | head | cut -d":" -f2| cut -d" " -f1'
		ip_buffer = os.popen('%s' % comando, 'r')
		ip_aux = ip_buffer.read()
		pattern_ip = re.compile('(\d+.\d+.\d+.\d+)')
		ip = re.findall(pattern_ip, ip_aux)
		return ip[0]

	##\fn comprobar_Conexion_Cliente(self, ip, mac)
	# \brief Funcion que comprueba si un cliente se encuentra conectado a la red y actualiza su estado 
	# en la lista de clientes conocidos. Tambien se compreba, en caso de tener conexion a la red, el estado
	# de JClic en el cliente.
	# \param[in] self Instancia de la clase que realiza la llamada
	# \param[in] ip Direccion IP del cliente que se desea comprobar su conexion
	# \param[in] mac Direccion MAC del cliente que se desea comprobar su conexion
	#def comprobar_Conexion_Cliente(self, ip, mac, inicial):
	def comprobar_Conexion_Cliente(self, ip, mac):
		# Se comprueba la conexion con el cliente
		sin_conexion = False
		imagen_conex = gtk.Image()
		try:
			hostname, aliaslist, ipaddrlist = socket.gethostbyaddr(ip)
		except socket.herror, msg:
			sin_conexion = True
		
		if sin_conexion == False:
			imagen_conex = gtk.gdk.pixbuf_new_from_file('%simg/network_local_icon.png' % self.PATH)
		else:
			imagen_conex = gtk.gdk.pixbuf_new_from_file('%simg/no_network_local_icon.png' % self.PATH)

		# Se actualiza su estado en la lista de clientes conocidos
		actualizado = False
		treeiter = self.listado.get_iter_first()
		while actualizado == False and treeiter != None:
			ip_actual = self.listado.get_value(treeiter, 4)
			if ip == ip_actual:
				self.listado.set_value(treeiter, 0, imagen_conex)
				if sin_conexion == False:
					self.listado.set_value(treeiter, 1, '1')
				else:
					self.listado.set_value(treeiter, 1, '0')
				actualizado = True
			else:
				treeiter = self.listado.iter_next(treeiter)
				if treeiter == None:
					break
		solicitador_estado = comunicaciones.EBCe()
		estado = solicitador_estado.enviar_mensaje_EBCe(ip, 'mensaje0003r\0', 60002)
		if estado == 1:
			self.set_Flag_Ejecucion_Remota(mac, 0)
	
	##\fn editar_cliente(self, widget, data = None)
	# \brief Funcion asociada al boton Editar de la aplicacion para abrir el dialogo de edicion
	# y realizar las modificaciones necesarias
	# \param[in] self Instancia de la clase que realiza la llamada
	# \param[in] widget Widget asociado al evento
	# \param[in] data Datos pasado a la duncion
	def editar_cliente(self, widget, data = None):
		self.window.set_sensitive(False)
		(model, fila_iterator) = self.seleccion_vista.get_selected()
                if fila_iterator != None:
                        sel_mac = model.get_value(fila_iterator, 6)
                        sel_ip = model.get_value(fila_iterator, 4)
                        sel_info = model.get_value(fila_iterator, 5)
                else: 
			print "Nada selecionado"
			self.window.set_sensitive(True)
			return 1
		print "%s" % sel_mac
		print "%s" % sel_ip
		print "%s" % sel_info
		editor = editor_Info_Cliente(self.window, self.gestor_lista, self.listado, sel_mac, sel_ip, sel_info)

	##\fn ejecutar_comprobacion(self)
	# \brief Funcion que ejecuta las comprobaciones de conexionado con los clientes
	# \param[in] self Instancia de la clase que realiza la llamada al metodo
	def ejecutar_comprobacion(self):
		while self.ejecutandose == True:
			clientes = self.gestor_lista.get_lista()
			if self.ejecutandose == True:	
				for cliente in clientes:
					datos_cliente = cliente.split(';;')
					self.comprobar_Conexion_Cliente(datos_cliente[1], datos_cliente[0])
			time.sleep(30.0)

	##\fn ejecutar_todos_clientes(self)
	# \brief Funcion encargada de enviar el mensaje de ejecucion de jclic player con informes
	# a todos los clientes que se encuentren conectados y tengan activado el demonio de informes
	# de jclic player
	# \param[in] self Instancia de la clase que realiza la llamada al metodo
	# \param[in] widget Widget asociado al evento
	# \param[in] data Datos pasados a la funcion
	def ejecutar_todos_clientes(self, widget, data = None):
		completado = False
		treeiter = self.listado.get_iter_first()
		while completado == False and treeiter != None:
			estado_jclic = self.listado.get_value(treeiter, 3)
			if estado_jclic == '1':
				ip_cliente = self.listado.get_value(treeiter, 4)
				self.enviar_ejecucion_cliente(ip_cliente)
			treeiter = self.listado.iter_next(treeiter)
			if treeiter == None:
				completado = True

	##\fn enviar_ejecucion_cliente(self, ip)
	# \brief Funcion que envia el mensaje de ejecucion a un cliente
	# \param[in] self Instancia de la clase que realiza la llamada al metodo
	# \param[in] ip Direccion IP del cliente al que se envia el mensaje
	def enviar_ejecucion_cliente(self, ip):
		emisor = comunicaciones.EBCe()
		emisor.enviar_mensaje_EBCe(ip, 'mensaje0006r\0', 60002)

	##\fn ejecutar_cliente_sel(self, widget, data = None)
	# \brief Funcion encargada de enviar el mensaje de ejecucion de jclic player con informes
	# al cliente seleccionado de la lista
	# \param[in] self Instancia de la clase que realiza la llamada al metodo
	# \param[in] widget Widget asociado al evento
	# \param[in] data Datos pasados a la funcion
	def ejecutar_cliente_sel(self, widget, data = None):
		(model, fila_iterator) = self.seleccion_vista.get_selected()
                if fila_iterator != None:
                        ip_cliente = model.get_value(fila_iterator, 4)
			self.enviar_ejecucion_cliente(ip_cliente)
                else: 
			print "Nada selecionado"
	
	##\fn borrar(self, widget, data = None)
	# \brief Funcion encargada de borrar elementos de la lista de clientes conocidos, tanto en el archivo
	# de clientes como en la interfaz grafica. Permite borrar un elemento o la lista entera, dependiendo 
	# de la etiqueta que tenga asignada el widget asociado a la llamada del metodo
	# \param[in] self Instancia de la clase que realiza la llamada al metodo
	# \param[in] widget Widget asociado al evento
	# \param[in] data Datos pasados a la funcion
	def borrar(self, widget, data = None):
		confirmacion = gtk.Dialog('Confirmar')
		confirmacion.add_button('Cancelar', gtk.RESPONSE_CANCEL)
		confirmacion.add_button('Aceptar', gtk.RESPONSE_OK)
		if widget.get_label() == 'Borrar lista':
			confirmacion.connect('response', self.borrar_lista_completa, None)
			msg = 'La lista de clientes conocidos sera eliminada completamente'
		elif widget.get_label() == 'Borrar seleccionado':
			confirmacion.connect('response', self.borrar_cliente, None)
			msg = 'El cliente seleccionado sera eliminado de la lista'
		else:
			print "Opcion incorrecta"
		mensaje = gtk.Label(msg)
		mensaje.set_justify(gtk.JUSTIFY_CENTER)
		confirmacion.vbox.pack_start(mensaje, True, True, 0)
		confirmacion.set_size_request(420, 150)
		mensaje.show()
		confirmacion.run()

	##\fn borrar_lista_completa(self, widget, response_id, data = None)
	# \brief Funcion que borra la lista completa de clientes conocidos
	# \param[in] self Instancia de la clase que realiza la llamada al metodo
	# \param[in] widget Widget asociado al evento
	# \param[in] response_id Respuesta obtenida en el evento
	# \param[in] data Datos pasados al metodo
	def borrar_lista_completa(self, widget, response_id, data = None):
		if response_id == -5:
			self.gestor_lista.borrar_lista()
			self.cargar_listado()
		widget.hide()
			
	##\fn borrar_cliente(self, widget, response_id, data = None)
	# \brief Funcion que borra un cliente de la lista de clientes conocidos
	# \param[in] self Instancia de la clase que realiza la llamada al metodo
	# \param[in] widget Widget asociado al evento
	# \param[in] response_id Respuesta obtenida en el evento
	# \param[in] data Datos pasados al metodo
	def borrar_cliente(self, widget, response_id, data = None):
		if response_id == -5:
			(model, fila_iterator) = self.seleccion_vista.get_selected()
	                if fila_iterator != None:
        	                mac_cliente = model.get_value(fila_iterator, 4)
				self.gestor_lista.borrar_cliente_mac(mac_cliente)
				self.listado.remove(fila_iterator)
			else:
				print "Nada seleccionado"
		widget.hide()
	
	##\fn mostrar_ayuda(self, widget, data = None)
	# \brief Funcion que muestra la ventana de ayuda
	# \param[in] self Instancia de la clase que realiza la llamada al metodo
	# \param[in] widget Widget asociado al evento
	# \param[in] data Datos pasados al metodo
	def mostrar_ayuda(self, widget, data = None):
		ayuda = jclic_Ayuda()

	##\fn mostrar_informes(self)
	# \brief Funcion que ejecuta un navegador web con la pagina de informes del servidor de contenidos
	# \param[in] self Instancia de la clase que realiza la llamada
	# \param[in] widget Widget asociado al evento
	# \param[in] data Datos pasados al metodo
	def mostrar_informes(self, widget, data = None):
		os.spawnlp(os.P_NOWAIT, 'firefox', 'firefox', '--new-window', 'http://localhost:9000/')	

##\class editor_Info_Cliente
# \brief Clase para la edicion de la informacion adicional de un cliente conocido
# \version 1.2.2-1
# \date 09/01/2008
class editor_Info_Cliente:
	##\fn __init__(self, win, lista, mac, ip, info)
	# \brief Constructor de la clase para crear la interfaz para la edicion de la informacion de los clientes
	# \param[in] self Instancia de la clase que realiza la llamada
	# \param[in] win Referencia a la ventana del gestor de clientes del servidor de informes
	# \param[in] lista_fich Referencia al gestor de lista utilizado en la aplicacion
	# \param[in] lista_graf Referencia al listado grafico de clientes de la aplicacion
	# \param[in] mac Direccion MAC del cliente seleccionado para modificar su informacion
	# \param[in] ip Direccion IP del cliente seleccionado para modificar su informacion
	# \param[in] info Informacion adicional del cliente que se desea modificar
	def __init__(self, win, lista_fich, lista_graf, mac, ip, info):
		self.PATH = '/usr/local/jclic/'
		self.window_padre = win
		self.gestor_lista = lista_fich
		self.listado = lista_graf
	
		# Se crea la ventana y el contenedor del cuadro de informacion
                self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
                self.window.set_position(gtk.WIN_POS_CENTER)
                self.window.set_keep_above(True)
                self.window.set_title("Servidor de Informes JClic - Información del receptor")
                self.window.set_icon_from_file('%simg/jclic_icon.png' % self.PATH)
                self.window.set_border_width(10)
                self.window.set_resizable(False)
                self.window.connect("delete_event", self.delete_event)
                self.window.connect("destroy", self.destroy)
                self.vbox = gtk.VBox(False, 30)
                self.panel = gtk.Fixed()
                self.vbox.pack_start(self.panel, True, True, 0)
                self.window.add(self.vbox)

		# Cabecera de la ventana
                self.imagen_aviso = gtk.Image()
                self.imagen_aviso.set_from_file('%simg/informacion.png' % self.PATH)
                self.panel.put(self.imagen_aviso, 130, 10)
                self.et_aviso = gtk.Label()
                self.et_aviso.set_markup("<b>Información del receptor</b>")
                self.et_aviso.set_use_markup(True)
                self.panel.put(self.et_aviso, 190, 25)

		# Informacion del receptor junto con los apartados modificables
                self.et_ip = gtk.Label()
                self.et_ip.set_markup("<b>Dirección IP:</b>  %s" % ip)
                self.et_ip.set_use_markup(True)
                self.panel.put(self.et_ip, 10, 90)
                self.et_mac = gtk.Label()
                self.et_mac.set_markup("<b>Dirección MAC:</b>  %s" % mac)
                self.et_mac.set_use_markup(True)
                self.panel.put(self.et_mac, 10, 120)
                self.et_info = gtk.Label()
                self.et_info.set_markup("<b>Información:</b>")
                self.et_info.set_use_markup(True)
                self.panel.put(self.et_info, 10, 160)
                self.in_info = gtk.Entry(50)
                self.in_info.set_size_request(200, 28)
                self.in_info.set_text(info)
                self.panel.put(self.in_info, 120, 156)
                self.et_max = gtk.Label("(max. 50 caracteres)")
                self.panel.put(self.et_max, 330, 160)
                self.et_nota = gtk.Label()
                self.et_nota.set_markup("<i>Nota: es posible modificar la información adicional que se muestra por\ncada cliente conocido, esta información puede ayudar al profesorado\na identificar facilmente los ordenadores dentro del aula. Para introducir\nla información basta con escribirla en el cuadro de texto anterior y\npulsar sobre el botón \"Aceptar\" para guardar el cambio.</i>")
                self.et_nota.set_use_markup(True)
                self.panel.put(self.et_nota, 10, 200)

		# Botonera de la ventana
                self.boton_aceptar = gtk.Button("Aceptar")
                self.boton_aceptar.connect("clicked", self.aplicar_cambio_cliente, mac)
                self.boton_aceptar.set_size_request(90,30)
                self.panel.put(self.boton_aceptar, 400, 300)
                self.boton_cancelar = gtk.Button("Cancelar")
                self.boton_cancelar.connect("clicked", self.delete_event)
                self.boton_cancelar.set_size_request(90,30)
                self.panel.put(self.boton_cancelar, 300, 300)

		# Dibujar la ventana
                self.window.set_size_request(500, 340)
                self.imagen_aviso.show()
                self.et_aviso.show()
                self.et_ip.show()
                self.et_mac.show()
                self.et_info.show()
                self.in_info.show()
                self.et_max.show()
                self.et_nota.show()
                self.boton_aceptar.show()
                self.boton_cancelar.show()
                self.panel.show()
                self.vbox.show()
                self.window.show()

	##\fn delete_event(self, widget, data = None)
	# \brief Funcion para terminar la ejecucion de la ventana de edicion de informacion
	# \param[in] self Instancia de clase que realiza la llamada
	# \param[in] widget Widget asociado al evento
	# \param[in] data datos pasados a la funcion	
	def delete_event(self, widget, data = None):
		self.window_padre.set_sensitive(True)
		self.window.destroy()

	##\fn destroy(self, widget, data = None)
	# \brief Funcion para terminar la ejecucion de la ventana de edicion de informacion	
	# \param[in] self Instancia de clase que realiza la llamada
	# \param[in] widget Widget asociado al evento
	# \param[in] data datos pasados a la funcion	
	def destroy(self, widget, data = None):
		self.window_padre.set_sensitive(True)
		self.window.destroy()

	##\fn aplicar_cambio_cliente(self, widget, mac, ip)
	# \brief Funcion para realizar la modificacion de la informacion adicional en el cliente
	# \param[in] self Instancia de la clase que realiza la llamada
	# \param[in] widget Widget asociado al evento
	# \param[in] mac Direccion MAC del cliente al que se realiza la modificacion
	def aplicar_cambio_cliente(self, widget, mac):
		self.gestor_lista.mod_info_cliente(mac, self.in_info.get_text())
		actualizado = False
                treeiter = self.listado.get_iter_first()
                while actualizado == False:
	                mac_actual = self.listado.get_value(treeiter, 6)
                        if mac_actual == mac:
        	                self.listado.set_value(treeiter, 5, self.in_info.get_text())
                                actualizado = True
                        else:
                                treeiter = self.listado.iter_next(treeiter)
	                        if treeiter == None:
        	                        break
		self.window_padre.set_sensitive(True)
		self.window.destroy()

##\class mensaje_Aviso
# \brief Clase para la creacion y publicacion de mensajes de aviso por pantalla
# \version 1.2.2-1
# \date 09/01/2008
class mensaje_Aviso:
	##\fn __init__(self, msg)
	# \brief Constructor de la clase. Crea una ventana de aviso con el mensaje correspondiente
	# \param[in] self Instancia de la clase que realiza la llamada
       	# \param[in] msg Mensaje que se va a publicar en el aviso
        def __init__(self, msg):
		self.PATH = '/usr/local/jclic/'
                self.cadena = msg

                # Se prepara el mensaje de entrada para su publicacion
                self.indice_linea = self.preparar_mensaje()

                # Ventana del mensaje y contenedor
                self.warning_window = gtk.Window(gtk.WINDOW_TOPLEVEL)
                self.warning_window.set_position(gtk.WIN_POS_CENTER)
                self.warning_window.set_title("Servidor de Informes JClic - Aviso")
                self.warning_window.set_icon_from_file('%simg/jclic_icon.png' % self.PATH)
                self.warning_window.set_border_width(10)
                self.warning_window.set_resizable(False)
                self.warning_window.connect("delete_event", self.delete_event)
                self.warning_window.connect("destroy", self.destroy)
                self.vbox = gtk.VBox(False, 30)
                self.panel = gtk.Fixed()
                self.vbox.pack_start(self.panel, True, True, 0)
                self.warning_window.add(self.vbox)


                # Cabecera de la ventana de aviso
                self.imagen_aviso = gtk.Image()
                self.imagen_aviso.set_from_file('%simg/warning.png' % self.PATH)
                self.panel.put(self.imagen_aviso, 130, 10)
                self.et_aviso = gtk.Label()
                self.et_aviso.set_markup("<b>Mensaje de aviso</b>")
                self.et_aviso.set_use_markup(True)
                self.panel.put(self.et_aviso, 190, 25)

                # Mensaje adaptado
                self.mensaje = gtk.Label()
                if msg != None:
                        self.mensaje.set_text("%s" % self.cadena)
                self.mensaje.set_justify(gtk.JUSTIFY_CENTER)
                self.panel.put(self.mensaje, 10, 70)


                # Botonera de la ventana
                # El mensaje es dinamico, tamano de la ventana y de la colocacacion de sus elementos
                # depende del numero de lineas del mensaje, indicado por la variable self.indice_linea
                self.boton = gtk.Button("Aceptar")
                self.boton.connect("clicked", self.delete_event, None)
                self.boton.set_size_request(90,30)
                altura_bot = 120 + (12*self.indice_linea)
                self.panel.put(self.boton, 400, altura_bot)


                # Dibujar la ventana de aviso
                altura = 160 + (12*self.indice_linea)
                self.warning_window.set_size_request(500, altura)
                self.imagen_aviso.show()
                self.et_aviso.show()
                self.mensaje.show()
                self.boton.show()
                self.panel.show()
                self.vbox.show()
                self.warning_window.show()

	##\fn delete_event(self, widget, data = None)
	# \brief Funcion para terminar la ejecucion de la ventana de aviso
	# \param[in] self Instancia de la clase que realiza la llamada al metodo
	# \param[in] widget Widget asociado al evento
	# \param[in] data Datos pasados a la funcion
	def delete_event(self, widget, data = None):
                self.warning_window.destroy()

	##\fn destroy(self, widget, data = None)
	# \brief Funcion para terminar la ejecucion de la ventana de aviso
	# \param[in] self Instancia de la clase que realiza la llamada al metodo
	# \param[in] widget Widget asociado al evento
	# \param[in] data Datos pasados a la funcion
        def destroy(self, widget, data = None):
                self.warning_window.destroy()

	##\fn preparar_mensaje(self)
        # \brief Funcion encargada de preparar el mensaje que se publicar
	# \param[in] self Instancia de la clase que realiza la llamada al metodo
	# \return Numero de lineas que compone el mensaje de aviso
	# \retval num_lin Numero de lineas que compone el mensaje de aviso
        def preparar_mensaje(self):
                num_car = len(self.cadena)
                act_car = 69  # numero de caracteres admitidos por cada linea
                num_lin = 1
                while act_car <= num_car:
                        while self.cadena[act_car] != ' ': # cada 69 caractreres del mensaje se introduce un salto de linea
                                act_car = act_car - 1
                        self.cadena = self.cadena[:act_car] + '\n' + self.cadena[act_car+1:]
                        num_lin = num_lin + 1
                        act_car = act_car + 70
                return num_lin

##\class jclic_Player_Informes
# \brief Clase para crear la interfaz de JClic Player con soporte para informes
# \version 1.2.2-1
# \date 17/01/2008
class jclic_Player_Informes:
	##\fn __init__(self, win)
	# \brief Constructor de la clase encargade de crear la interfaz de usuario
	# \param[in] self Instancia de la clase que realiza la llamada
	# \param[in] win Referencia a la ventana base de la aplicacion
	def __init__(self, win):
		self.PATH = '/usr/local/jclic/'
		self.window_base = win
		
		self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
		self.window.set_border_width(10)
		self.window.set_icon_from_file('%simg/jclic_icon.png' % self.PATH)
		self.window.set_title('Servidor de informes JClic - Conectar a servidor de informes')
		self.window.set_resizable(False)
		self.window.set_size_request(570, 250)
		self.window.connect("delete_event", self.delete_event)
                self.window.connect("destroy", self.destroy)
		self.box = gtk.VBox(False, 10)
		self.window.add(self.box)
		self.box.show()
		
		self.mensaje = gtk.Label('Desde esta aplicacion puede conectarse al servidor de informes de JClic del aula.\nEsta aplicacion no ejecutara JClic Player en este ordenador, simplemente notificara\nal servidor de informes que se encuentra disponible para ejecutar JClic Player cuando\neste los solicite remotamente.')
		self.box.pack_start(self.mensaje, False, False, 0)
		self.mensaje.show()
		
		#Cuadro para introducir la direccion IP de emision
                self.dirip_panel = gtk.Fixed()
                self.dirip_panel.set_size_request(390, 110)
                self.et_ip = gtk.Label()
                self.et_ip.set_label("Introduzca la dirección IP del servidor de informes de JClic:")
                self.dirip_panel.put(self.et_ip, 10, 10)

                self.dirip_panel.show()
                self.et_ip.show()
                # Se crean 4 widgets de entrada de texto, uno por cada octeto de la direccion IP
                # Se prefijan los dos primeros octetos a 192 y 168 respectivamente para facilitar su uso
                # estos parametros prefijados se pueden modificar
                # Este formato es mas adecuado para comprobar si la direccion IP es valida
                self.in_ip_1 = gtk.Entry(3)
                self.in_ip_1.set_size_request(40, 24)
                self.in_ip_1.set_alignment(1.0)
                self.in_ip_1.set_text("192")
                self.et_ip_1 = gtk.Label(".")
                self.dirip_panel.put(self.et_ip_1, 53, 68)
                self.dirip_panel.put(self.in_ip_1, 10, 60)

                self.in_ip_2 = gtk.Entry(3)
                self.in_ip_2.set_size_request(40, 24)
                self.in_ip_2.set_alignment(1.0)
                self.in_ip_2.set_text("168")
                self.et_ip_2 = gtk.Label(".")
                self.dirip_panel.put(self.et_ip_2, 103, 68)
                self.dirip_panel.put(self.in_ip_2, 60, 60)

		self.in_ip_3 = gtk.Entry(3)
                self.in_ip_3.set_size_request(40, 24)
                self.in_ip_3.set_alignment(1.0)
                self.et_ip_3 = gtk.Label(".")
                self.dirip_panel.put(self.et_ip_3, 153, 68)
                self.dirip_panel.put(self.in_ip_3, 110, 60)

                self.in_ip_4 = gtk.Entry(3)
                self.in_ip_4.set_size_request(40, 24)
                self.in_ip_4.set_alignment(1.0)
                self.dirip_panel.put(self.in_ip_4, 160, 60)

                self.in_ip_1.show()
                self.et_ip_1.show()
                self.in_ip_2.show()
                self.et_ip_2.show()
                self.in_ip_3.show()
                self.et_ip_3.show()
                self.in_ip_4.show()

		self.box.pack_start(self.dirip_panel, False, False, 0)

		# Botonera de acciones
                # Se incluyen dos botones
                # - Aceptar para conectar con el emisor
                # - Cancelar para terminar la ejecucion de la aplicacion y no conectar
                self.botonera_panel = gtk.Fixed()
		self.box.pack_start(self.botonera_panel, False, False, 0)
                self.conectar = gtk.Button("Conectar")
		self.conectar.connect('clicked', self.conectar_servidor, None)
                self.cancelar_conexion = gtk.Button("Cancelar")
		self.cancelar_conexion.connect('clicked', self.delete_event)
                self.botonera_panel.put(self.conectar, 450, 0)
                self.botonera_panel.put(self.cancelar_conexion, 340, 0)
                self.conectar.set_size_request(100, 30)
                self.cancelar_conexion.set_size_request(100, 30)
                self.conectar.show()
                self.cancelar_conexion.show()
                self.botonera_panel.show()
		
		self.window.show()

	##\fn delete_event(self, widget, data = None)
	# \brief Funcion para terminar la ejecucion de la ventana jclic player informes
	# \param[in] self Instancia de clase que realiza la llamada
	# \param[in] widget Widget asociado al evento
	# \param[in] data datos pasados a la funcion	
	def delete_event(self, widget, data = None):
		self.window_base.set_sensitive(True)
		self.window.destroy()

	##\fn destroy(self, widget, data = None)
	# \brief Funcion para terminar la ejecucion de la ventana jclic player informes	
	# \param[in] self Instancia de clase que realiza la llamada
	# \param[in] widget Widget asociado al evento
	# \param[in] data datos pasados a la funcion	
	def destroy(self, widget, data = None):
		self.window_base.set_sensitive(True)
		self.window.destroy()

	##\fn conectar_servidor(self, widget, data = None)
	# \brief Funcion que conecta con el servidor de informes
	# \param[in] self Instancia de la clase que realiza la llamada al metodo
	# \param[in] widget Widget asociado al evento
	# \param[in] data Datos pasados a la funcion
	def conectar_servidor(self, widget, data = None):
                if self.in_ip_3.get_text() != '' and self.in_ip_4.get_text() != '':
                        in_ip_1_num = string.atoi(self.in_ip_1.get_text())
                        in_ip_2_num = string.atoi(self.in_ip_2.get_text())
                        in_ip_3_num = string.atoi(self.in_ip_3.get_text())
                        in_ip_4_num = string.atoi(self.in_ip_4.get_text())
                        if in_ip_3_num >= 0 and in_ip_3_num <= 255 and in_ip_4_num >= 0 and in_ip_4_num <= 255 and in_ip_1_num == 192 and in_ip_2_num == 168:
                                # Reconstruccion de IP y se intenta conectar con el emisor
                	        ip = self.in_ip_1.get_text() + "." + self.in_ip_2.get_text() + "." + self.in_ip_3.get_text() + "." + self.in_ip_4.get_text()
				conector = comunicaciones.EBCe()
				estado = conector.enviar_mensaje_EBCe(ip, 'mensaje0001r\0', 60001)
				if estado == 1:
                                	aviso = mensaje_Aviso("No ha sido posible conectar con el servidor de informes de JClic. Es posible que la dirección IP introducida no se corresponda con un servidor de informes, que el servidor de informes ya no se encuentre activado o que este ordenador no se encuentre conectado a la red. En cualquier caso, revise su configuración e intente de nuevo conectar con el servidor de informes de JClic.")
				else:
					self.window_base.set_sensitive(True)
					self.window.destroy()
			else:
                                aviso = mensaje_Aviso("La dirección IP introducida no es correcta, por favor revise los datos introducidos y vuelva a intentar conectar de nuevo con el servidor de informes de JClic.")
                else:
                	aviso = mensaje_Aviso("La dirección IP introducida no es correcta, por favor revise los datos introducidos y vuelva a intentar conectar de nuevo con el servidor de informes de JClic.")

##\class jclic_Ayuda
# \brief Clase para crear la ventana de ayuda
# \version 1.2.2-1
# \date 17/01/2008
class jclic_Ayuda:
	##\fn __init__(self)
	# \brief Constructor de la clase que crea la ventana de ayuda
	# \param[in] self Instancia de la clase que realiza la llamada
	def __init__(self):
		self.PATH = '/usr/local/jclic/'
	
		self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
		self.window.set_border_width(10)
		self.window.set_icon_from_file('%simg/jclic_icon.png' % self.PATH)
		self.window.set_title('Servidor de informes JClic - Ayuda')
		self.window.set_resizable(False)
		self.window.set_size_request(430, 420)
		self.window.connect("delete_event", self.delete_event)
                self.window.connect("destroy", self.destroy)
		self.box = gtk.VBox(False, 10)
		self.window.add(self.box)
		self.box.show()
	
		self.panel = gtk.Fixed()
		self.box.pack_start(self.panel, False, False, 0)
		self.panel.show()
		
		self.mensaje = gtk.Label()
		self.mensaje.set_markup('<b>Iconos de estado utilizados en la aplicación</b>')
                self.mensaje.set_use_markup(True)
		self.mensaje.show()
		self.panel.put(self.mensaje, 40, 10)

		self.frame_iconos = gtk.Frame()
		self.frame_iconos.set_size_request(390, 290)
		self.fixed_iconos = gtk.Fixed()
		self.frame_iconos.add(self.fixed_iconos)
		self.frame_iconos.show()
		self.fixed_iconos.show()
		self.panel.put(self.frame_iconos, 10, 50)

		self.imagen_red = gtk.Image()
		self.imagen_red.set_from_file('%simg/network_local_icon.png' % self.PATH)
		self.et_red = gtk.Label('El cliente se encuentra conectado a la red')
		self.fixed_iconos.put(self.imagen_red, 20, 10)
		self.fixed_iconos.put(self.et_red, 60, 10)
		self.imagen_red.show()
		self.et_red.show()
		
		self.imagen_no_red = gtk.Image()
		self.imagen_no_red.set_from_file('%simg/no_network_local_icon.png' % self.PATH)
		self.et_no_red = gtk.Label('El cliente no se encuentra conectado a la red o se\nencuentra apagado actualmente')
		self.fixed_iconos.put(self.imagen_no_red, 20, 50)
		self.fixed_iconos.put(self.et_no_red, 60, 50)
		self.imagen_no_red.show()
		self.et_no_red.show()
		
		self.imagen_no_jclic = gtk.Image()
		self.imagen_no_jclic.set_from_file('%simg/jclicplayer_cerrar_icon.png' % self.PATH)
		self.et_no_jclic = gtk.Label('El cliente no está preparado para ejecutar JClic\nPlayer con soporte de informes')
		self.fixed_iconos.put(self.imagen_no_jclic, 20, 110)
		self.fixed_iconos.put(self.et_no_jclic, 60, 110)
		self.imagen_no_jclic.show()
		self.et_no_jclic.show()
		
		self.imagen_si_jclic = gtk.Image()
		self.imagen_si_jclic.set_from_file('%simg/jclicejecutando_icon.png' % self.PATH)
		self.et_si_jclic = gtk.Label('El cliente está preparado para ejecutar JClic\nPlayer con soporte de informes')
		self.fixed_iconos.put(self.imagen_si_jclic, 20, 170)
		self.fixed_iconos.put(self.et_si_jclic, 60, 170)
		self.imagen_si_jclic.show()
		self.et_si_jclic.show()
		
		self.imagen_uso_jclic = gtk.Image()
		self.imagen_uso_jclic.set_from_file('%simg/jclicplayerinforme_icon.png' % self.PATH)
		self.et_uso_jclic = gtk.Label('El cliente está realizando actualmente una\nactividad con JClic Player con soporte de informes')
		self.fixed_iconos.put(self.imagen_uso_jclic, 20, 230)
		self.fixed_iconos.put(self.et_uso_jclic, 60, 230)
		self.imagen_uso_jclic.show()
		self.et_uso_jclic.show()

		self.boton_cerrar = gtk.Button('Cerrar')
		self.boton_cerrar.set_size_request(150, 30)
		self.panel.put(self.boton_cerrar, 130, 360)
		self.boton_cerrar.connect('clicked', self.delete_event)
		self.boton_cerrar.show()
		
		self.window.show()

	##\fn delete_event(self, widget, data = None)
	# \brief Funcion para terminar la ejecucion de la ventana de ayuda
	# \param[in] self Instancia de clase que realiza la llamada
	# \param[in] widget Widget asociado al evento
	# \param[in] data datos pasados a la funcion	
	def delete_event(self, widget, data = None):
		self.window.destroy()

	##\fn destroy(self, widget, data = None)
	# \brief Funcion para terminar la ejecucion de la ventana de ayuda	
	# \param[in] self Instancia de clase que realiza la llamada
	# \param[in] widget Widget asociado al evento
	# \param[in] data datos pasados a la funcion	
	def destroy(self, widget, data = None):
		self.window.destroy()

